#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
function pcAfterFn(projectDir, projectName, webTitle) {
    //处理token名称
    replaceContent(projectDir + '/src/configure/index.js', '{PROJECT_NAME}', projectName);
    replaceContent(projectDir + '/src/views/auth/login.vue', '{PROJECT_NAME}', projectName);
    replaceContent(projectDir + '/src/views/layout/index.vue', '{PROJECT_NAME}', projectName);
    //处理浏览器标题名称
    replaceContent(projectDir + '/.env', '{WEB_TITLE_NAME}', webTitle);
}
function h5AfterFn(projectDir, projectName, webTitle) {
    //处理token名称
    replaceContent(projectDir + '/src/views/login/index.vue', '{PROJECT_NAME}', projectName);
    //处理浏览器标题名称
    replaceContent(projectDir + '/.env', '{WEB_TITLE_NAME}', webTitle);
}
const tplMap = {
    pc: {
        url: 'https://codeup.aliyun.com/60627e6e6cc98419b95d4a2a/framework/front-template.git',
        afterFn: pcAfterFn
    },
    h5: {
        url: 'https://codeup.aliyun.com/60627e6e6cc98419b95d4a2a/framework/h5-template.git',
        afterFn: h5AfterFn
    }
};
const commander_1 = require("commander");
const mkdirp_1 = __importDefault(require("mkdirp"));
const ora_1 = __importDefault(require("ora"));
const path = require('path');
const download = require('download-git-repo');
const fsExtra = require('fs-extra');
const inquirer = require('inquirer');
const { spawn } = require('child_process');
const updateNotifier = require('update-notifier');
const commandSpawn = (...args) => {
    return new Promise(resolve => {
        const childProcess = spawn(...args);
        // 显示子进程的控制台打印信息
        childProcess.stdout.pipe(process.stdout);
        childProcess.stderr.pipe(process.stderr);
        childProcess.on('close', resolve);
    });
};
const packageJson = require('../../package.json');
updateNotifier({ pkg: packageJson, updateCheckInterval: 1000 }).notify();
const tip = (0, ora_1.default)();
function replaceContent(filePath, target, value) {
    fsExtra.readFile(filePath, 'utf8').then(function (fileData) {
        fsExtra.outputFile(filePath, fileData.split(target).join(value));
    });
}
commander_1.program.version(packageJson.version, '-v, --version');
commander_1.program
    .command("create <app-name>")
    .description(`使用 ${packageJson.name} 初始化项目`)
    .action(projectName => {
    tip.info(`开始初始化项目[${projectName}]`);
    const projectDir = path.join(process.cwd(), projectName); // 项目创建路径
    // 使用mkdirp避免一级一级创建目录
    (0, mkdirp_1.default)(projectDir).then((made) => __awaiter(void 0, void 0, void 0, function* () {
        if (made === undefined) {
            tip.fail('创建失败，存在同名目录');
            return;
        }
        else {
            inquirer.prompt([
                {
                    type: 'input',
                    name: 'webTitle',
                    message: '请输入项目名称(用作浏览器title展示)',
                },
                {
                    type: 'input',
                    name: 'serverAddress',
                    message: '请输入要访问的后台地址(默认：http://127.0.0.1:6891)'
                },
                {
                    type: 'list',
                    name: 'tplType',
                    message: '请选择需要初始化的模板类型',
                    default: 'pc',
                    choices: [
                        { value: 'pc', name: 'pc模板' },
                        { value: 'h5', name: 'h5模板' }
                    ]
                }
            ]).then(function (res) {
                return __awaiter(this, void 0, void 0, function* () {
                    const { webTitle, serverAddress, tplType } = res;
                    const { url: downloadTplUrl, afterFn } = tplMap[tplType];
                    if (!downloadTplUrl) {
                        tip.fail('未匹配到正确的模板地址，无法初始化');
                        return;
                    }
                    const spinner = (0, ora_1.default)(`正在下载项目模板，源地址：${downloadTplUrl}`).start();
                    download(`direct:${downloadTplUrl}`, projectDir, { clone: true }, function (err) {
                        return __awaiter(this, void 0, void 0, function* () {
                            if (err) {
                                spinner.fail();
                                tip.fail(err.message || '下载模板异常');
                            }
                            else {
                                spinner.start('正在处理项目文件');
                                afterFn && afterFn(projectDir, projectName, webTitle);
                                //处理package.json中的项目名
                                replaceContent(projectDir + '/package.json', 'PROJECT_NAME', projectName);
                                //处理token名称
                                replaceContent(projectDir + '/src/configure/http/index.js', '{PROJECT_NAME}', projectName);
                                replaceContent(projectDir + '/src/configure/router/index.js', '{PROJECT_NAME}', projectName);
                                //处理后台地址
                                replaceContent(projectDir + '/vue.config.js', '{SERVER_ADDRESS}', serverAddress || 'http://127.0.0.1:6891');
                                //处理浏览器标题名称
                                replaceContent(projectDir + '/.env', '{WEB_TITLE_NAME}', webTitle);
                                tip.succeed(`项目创建完毕，目录：${projectDir}`);
                                spinner.start('正在安装依赖');
                                try {
                                    const command = process.platform === 'win32' ? 'yarn.cmd' : 'yarn';
                                    yield commandSpawn(command, ['install'], { cwd: `./${projectName}/` });
                                    spinner.succeed('项目初始化完毕，请进入项目目录，启动项目查看！');
                                }
                                catch (e) {
                                    console.log(e.message);
                                    spinner.fail('安装依赖异常，请尝试进入项目目录，手动进行安装！');
                                }
                            }
                        });
                    });
                });
            });
        }
    }));
});
commander_1.program.parse(process.argv); //解析输入命令
